#!/bin/bash
#Copyright [2018 - 2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

#Move elastic 6.8 to opensearch

TMP_LOG_FILE="/tmp/cis_opensearch_prepare.log"
DATE_NOW=$(date +%s)
LOG_FILE="/var/opt/novell/log/cis/cis_opensearch_prepare_$DATE_NOW.log"

printandExit(){
  if [[ $1 -ne 0 ]]; then
    echo -e "Migrating data to OpenSearch failed." | tee -a "$TMP_LOG_FILE"
    if [ "$TMP_LOG_FILE" != "$LOG_FILE" ]; then
      mv "$TMP_LOG_FILE" "$LOG_FILE"
    fi
    exit 1
  else
    echo -e "Data from ElasticSearch was successfully transferred to OpenSearch. Cleanup is done by deleting outdated data." | tee -a "$TMP_LOG_FILE"
    if [ "$TMP_LOG_FILE" != "$LOG_FILE" ]; then
      mv "$TMP_LOG_FILE" "$LOG_FILE"
    fi
    exit 0
  fi
}

isElastic6_8() {
    elasticversion=""
    
    configFile="/opt/novell/cis/es/bin/es_service.sh"
    line='${ES_NAME} docker.elastic.co/elasticsearch/elasticsearch-oss:6.8.22 &'
    result=`grep "$line" "$configFile"`

    if [ -f $configFile ]; then
        echo "es_service file exists. " >> "$TMP_LOG_FILE" 2>&1
        if [ -z "$result" ]; then
            echo "Not a 6.8 elastic install " >> "$TMP_LOG_FILE" 2>&1                  
        else 
            echo "It is a 6.8 elastic install " >> "$TMP_LOG_FILE" 2>&1            
            elasticversion="6.8.0"            
        fi
    else
        echo "es_service file is not accessible " >> "$TMP_LOG_FILE" 2>&1        
    fi

    if [ -d /etc/opt/novell/cis/es ]; then
        echo "Directory /etc/opt/novell/cis/es is available " >> "$TMP_LOG_FILE" 2>&1
    else 
        echo "Directory /etc/opt/novell/cis/es is unavailable " >> "$TMP_LOG_FILE" 2>&1        
    fi

    if [ "$elasticversion" == "6.8.22" ]; then
        return 1
    else 
        return 0
    fi
}

cleanup() {
    config_Type=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' | tr -d '"' | xargs`
    MOVE_LOG_FILE="$1"

    if [ "$MOVE_LOG_FILE" == "" ]; then
       MOVE_LOG_FILE=$TMP_LOG_FILE
    fi

    if [ $config_Type == "standalone" ]; then
        systemctl stop oes-cis-elastic.service
        if [ $? -ne 0 ]; then 
                echo "Could not stop Elastic 6.8 service " >> "$MOVE_LOG_FILE" 2>&1         
                printandExit 1
        fi    
        systemctl disable oes-cis-elastic.service >> "$MOVE_LOG_FILE" 2>&1  
    fi   

    cleanupRemaining "$MOVE_LOG_FILE"
}

cleanupRemaining() {
    MOVE_LOG_FILE="$1"
    if [ "$MOVE_LOG_FILE" == "" ]; then
       MOVE_LOG_FILE=/tmp/cisopensearch 
    fi

    rm -rf /opt/novell/cis/es
    rm -rf /etc/opt/novell/cis/es
    #rm -rf /var/opt/novell/cis/elastic*

    rm /usr/lib/systemd/system/oes-cis-elastic.service >> "$MOVE_LOG_FILE" 2>&1  
    systemctl daemon-reload   

    #remove docker images
    for i in $(docker ps -a 2>/dev/null | grep "elastic" | awk '{print $1}');
    do
        docker stop $i >> "$MOVE_LOG_FILE" 2>&1  
        docker rm $i >> "$MOVE_LOG_FILE" 2>&1  
    done
    
    docker rmi $(docker images --filter "reference=docker.elastic.co/elasticsearch/elasticsearch-oss" -q) >> "$MOVE_LOG_FILE" 2>&1
    
    /usr/bin/docker rm elasticdump/elasticsearch-dump >> "$MOVE_LOG_FILE" 2>&1  
    docker rmi elasticdump/elasticsearch-dump >> "$MOVE_LOG_FILE" 2>&1
    printandExit 0
}

loadDump(){
    MOVE_LOG_FILE=$1
    /usr/bin/docker load -i /var/opt/novell/cis/elasticdump.tar.gz >> "$MOVE_LOG_FILE" 2>&1 
    npm install elasticdump -g >> "$MOVE_LOG_FILE" 2>&1 
}

startDump(){    
    MOVE_LOG_FILE=$1
    index=$2
    config_Type=`cat /etc/opt/novell/cis/configurationStatus.json | awk -F"[,:}]" '{for(i=1;i<=NF;i++){if($i~/\042'configType'\042/){print $(i+1)}}}' | tr -d '"' | xargs`

    if [ $config_Type == "standalone" ]; then
        status=`systemctl is-active oes-cis-opensearch.service`   
        if [ "$status" != "active" ]; then 
            echo "OpenSearch service is not running"          
            exit 1
        fi
        status=`systemctl is-active oes-cis-elastic.service`   
        if [ "$status" != "active" ]; then  
            echo "Elastic service 6.8 is not running"                         
            exit 1
        fi    
    fi

    # read data from config file
    BATCH=`grep "BATCH" /etc/opt/novell/cis/es/dumpconfig | cut -d '=' -f 2`
    CONCURRENCY=`grep "CONCURRENCY" /etc/opt/novell/cis/es/dumpconfig | cut -d '=' -f 2`

    ipconfig=`grep IP_CONFIG /etc/opt/novell/cis/config`
    localip=`echo $ipconfig | cut -d '"' -f2`

    #echo "Dump $1 $2 $3"
    if [ -z $index ]; then 
        getDocumentCount "1" "all"
    else 
        getDocumentCount "1" $index
    fi

    if [ -f /tmp/ciselasticcount$index ]; then
	    count=$(cat /tmp/ciselasticcount$index)
	    # copy the original number to another file
        cp /tmp/ciselasticcount$index /tmp/originalcount$index
        if [ -z $index ]; then
            echo "Number of documents to be migrated " $count >> "$MOVE_LOG_FILE" 2>&1 
        else
            echo "Number of documents to be migrated for index $index " $count >> "$MOVE_LOG_FILE" 2>&1 
        fi
    else
        count="-1"
    fi

    if [ $count != "-1" ]; then
        if [ -z $index  ]; then
            #echo "/usr/bin/docker run --rm -i -v elasticdump/elasticsearch-dump:latest --input http://$localip:19200 --output http://$localip:9200 --type=data --delete --support-big-int --limit ${BATCH} --concurrency ${CONCURRENCY}" 
            elasticdump --input https://$localip:9500 --output https://$localip:9400 --type=data --cert /etc/opt/novell/cis/certs/servercert.pem \
        --key /etc/opt/novell/cis/certs/serverkey.pem --cacert /etc/opt/novell/cis/certs/rootCAs/SSCert.pem --tlsAuth --delete --support-big-int --limit ${BATCH} --concurrency ${CONCURRENCY} >> "$MOVE_LOG_FILE" 2>&1
        else
            #echo "/usr/bin/docker run --rm -i -v elasticdump/elasticsearch-dump:latest --input http://$localip:19200/$index --output http://$localip:9200/$index --delete --type=data --support-big-int --limit ${BATCH} --concurrency ${CONCURRENCY}"
            elasticdump --input https://$localip:9500/$index --output https://$localip:9400/$index --type=data --cert /etc/opt/novell/cis/certs/servercert.pem \
        --key /etc/opt/novell/cis/certs/serverkey.pem --cacert /etc/opt/novell/cis/certs/rootCAs/SSCert.pem --tlsAuth --delete --support-big-int --limit ${BATCH} --concurrency ${CONCURRENCY} >> "$MOVE_LOG_FILE" 2>&1
        fi  
        
        # check if elastic copying is complete
        count="1"    
        iteration=1
            
        while [ $iteration -lt 5 ];
        do           
            rm /tmp/ciselasticcount$index
            if [ -z $index ]; then
                getDocumentCount "1" "all"
            else  
                getDocumentCount "1" $index
            fi

            if [ -f /tmp/ciselasticcount$index ]; then
                count=$(cat /tmp/ciselasticcount$index)
                echo "Getting count " $count " " $iteration  >> "$MOVE_LOG_FILE" 2>&1        
            else
                count="-1"
            fi
            if { [ "$count"="-1" ] || [ "$count"="0" ]; }; then 
                break
            fi
            iteration=$[iteration + 1]
        done

        if [ "$count"="0" ]; then   
            if [ -z $index ]; then   
                echo "Elastic data has been migrated successfully to OpenSearch. " | tee -a "$MOVE_LOG_FILE"
                echo "Navigate to CIS console and verify if the migrated data is available then perform cleanup." | tee -a "$MOVE_LOG_FILE"
                echo "Refer documentation for cleanup and further steps." | tee -a "$MOVE_LOG_FILE"
            else 
                echo "Elastic data from $index has been migrated successfully to OpenSearch. " | tee -a "$MOVE_LOG_FILE"
                echo "Migrate all indices to Opensearch and then run verify command. " | tee -a "$MOVE_LOG_FILE"
            fi
            
            exit 0
        else             
            PrintUnsuccessful $config_Type $MOVE_LOG_FILE
        fi
    else
        PrintUnsuccessful $config_Type $MOVE_LOG_FILE
    fi    
}

PrintUnsuccessful() {
    MOVE_LOG_FILE=$2
    echo "Elastic data has not migrated sucessfully. Please run the below command again." | tee -a "$MOVE_LOG_FILE"  
    if [ $1 == "standalone" ]; then
        echo "sh /opt/novell/cis/bin/elastic_move_data.sh migrate" | tee -a "$MOVE_LOG_FILE" 
    else
        echo "sh /opt/novell/cis/bin/cis_ext_service.sh migrate" | tee -a "$MOVE_LOG_FILE" 
    fi
    exit 1
}

stopDump() {
    MOVE_LOG_FILE=$1
    /usr/bin/docker stop elasticdump/elasticsearch-dump >> "$MOVE_LOG_FILE" 2>&1  
}

getDocumentCount() {
    elastic=$1
    docindex=$2

    ipconfig=`grep IP_CONFIG /etc/opt/novell/cis/config`
    localip=`echo $ipconfig | cut -d '"' -f2`

    if [ -z $docindex ]; then
        docindex="all"
    fi

    if [ "$elastic" = "0" ]; then
        /opt/novell/cis/bin/cismigrate --count --zk_uri $localip --elastic "false" --index $docindex
    else
        /opt/novell/cis/bin/cismigrate --count --zk_uri $localip --elastic "true" --index $docindex
    fi
}

migrate() {
    DATE_NOW=$(date +%s)
    MOVE_LOG_FILE="/var/opt/novell/log/cis/cis_opensearch_move_$DATE_NOW.log"

    index=$3
    #echo $2    

    loadDump $MOVE_LOG_FILE
    startDump $MOVE_LOG_FILE $index

    if [ $?==0] ; then
        exit 0
    else 
        exit 1
    fi
}

removeImages() {
    docker rmi $(docker images --filter "reference=docker.elastic.co/elasticsearch/elasticsearch-oss" -q) > /dev/null 2>&1
}

verify() {
    DATE_NOW=$(date +%s)
    MOVE_LOG_FILE="/var/opt/novell/log/cis/cis_opensearch_verify_$DATE_NOW.log"

    verifyIndices $MOVE_LOG_FILE
    
    if [ -f /tmp/originalcount ]; then 
      count=$(cat /tmp/originalcount)
    else 
      echo "Migration has not been executed for all indices as one command."
      echo "Exiting."
      exit 0
    fi

    echo "$count Documents were present in ElasticSearch."  | tee -a "$MOVE_LOG_FILE" 

    getDocumentCount "0" "all"

    if [ -f /tmp/cisopensearchcount ]; then 
      migratedcount=$(cat /tmp/cisopensearchcount)
      echo "$migratedcount Documents are present in OpenSearch."  | tee -a "$MOVE_LOG_FILE" 

      if [ $count == $migratedcount ]; then 
        echo "All $count documents from ElasticSearch have been moved to OpenSearch. Data migration to OpenSearch is successful." | tee -a "$MOVE_LOG_FILE" 
      else
        echo "$migratedcount out of $count documents from ElasticSearch have been moved to OpenSearch. To complete migration, refer documentation." | tee -a "$MOVE_LOG_FILE"         
      fi
    else
        echo "Verify operation has failed to get document count from OpenSearch"  | tee -a "$MOVE_LOG_FILE"   
    fi

    
}

verifyIndices() {
    MOVE_LOG_FILE=$1

    Indices=("reporting_fileinfo" "reporting_jobinfo" "reporting_cache" "insights_volume_summary" "insights_file_info" "insights_status_summary")
    for index in ${Indices[*]}; do
        if [ -f /tmp/originalcount$index ]; then 
            count=$(cat /tmp/originalcount$index)
            echo "$count Documents were present in ElasticSearch index $index."  | tee -a "$MOVE_LOG_FILE" 

            getDocumentCount "0" $index

            if [ -f /tmp/cisopensearchcount$index ]; then 
                migratedcount=$(cat /tmp/cisopensearchcount$index)
                echo "$migratedcount Documents are present in OpenSearch index $index."  | tee -a "$MOVE_LOG_FILE" 

                if [ $count == $migratedcount ]; then 
                    echo "All $count documents from ElasticSearch index $index have been moved to OpenSearch. Data migration of $index to OpenSearch is successful." | tee -a "$MOVE_LOG_FILE" 
                else
                    echo "$migratedcount out of $count documents from ElasticSearch $index have been moved to OpenSearch. To complete migration, refer documentation." | tee -a "$MOVE_LOG_FILE"         
                fi
            else
                echo "Verify operation has failed to get document count from OpenSearch"  | tee -a "$MOVE_LOG_FILE"   
            fi
        else 
            echo "Migration command has not been executed independently for $index."
            continue      
        fi        
    done
}

case "$1" in
    startDump)
        startDump
        ;;
    stopDump)
        stopDump
        ;;
    loadDump)
	loadDump
	;;
    getDocumentCount)
        getDocumentCount
        ;;
    migrate)
        migrate $0 $1 $2 $3
        ;;
    cleanup)
        cleanup
        ;;
    removeImages)
        removeImages
        ;;
    verify)
        verify
        ;;
esac



